import requests
import base64

#defining global variables, in this case API_URL and API_KEY
API_URL = "https://api.price2spy.com/rest/v1/"
API_KEY = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

#Defining a function for decoding the API_KEY
def decodeAPIKey(key):
    api_key_bytes = key.encode()
    encoded = base64.b64encode(api_key_bytes)
    return(encoded.decode('utf-8'))

#headers for the request, they determine the type and the API key needed
headers = {
    'accept': 'application/json',
    'Content-Type': 'application/json',
    'Authorization': 'Basic ' + decodeAPIKey(API_KEY),
}

#defining a function to get all urls for a specific product
def getAllUrls(productId):

    data = '{ "id": ' + str(productId) + ', "idMode": "ID", "searchMode": "LIKE"}'
    response = requests.post(API_URL + 'get-urls', headers=headers, data=data)
    urls = response.json()
    return urls

#defining a function to get all measurements for a specific URL

def getAllMeasurements(urlId):

    data = '{"urlId": ' + str(urlId) + '}'
    response = requests.post(API_URL + 'get-measurements', headers=headers, data=data)
    measurements = response.json()
    return measurements

#sending the request to get all products
data = '{ "active": true}'
response = requests.post(API_URL + 'get-products', headers=headers, data=data)



#deserializes the response so it becomes a dict 
#I wrapped this part in a try-catch block because it's the one that first raised an exception if the authentification didn't work as intended
#it can print whatever message it needs and be transparent as needed about it
try:
    allProducts = response.json()
except:
    print(response.content)


#loops through each product, prints it's name, calls the function for getting urls for that products, loops through each url and 
#prints address and all measurements for it. For this example, I wrote it to print the price and currency together but they can be separated if needed. 
#it's also in a try-catch block because it can raise an exception without it


try:
   for x in allProducts['product']:
        print("Product name: " + x.get('productName') + "\n")
        allUrls = getAllUrls(x.get('productId'))
        for y in allUrls['url']:
            print("URL: " + y.get('url') + "\n")
            allMeasurements = getAllMeasurements(y.get('urlId'))
            for z in allMeasurements['measurement']:
                print("Date checked: " + str(z.get('dateChecked')) + "\n")
                print("Price: " + str(z.get('price')['amount']) + ' ' + z.get('price')['currency'] + "\n")
                print("Available: " + str(z.get('available')) + "\n")
                print("Is on a promotion: " + str(z.get('isOnPromotion')) + "\n")
                print("Screenshot URL: " + str(z.get('screenshotUrl')) + "\n")

except NameError as err:
    print("Name error: {0}".format(err))
except:
    print("Error occured.")


""" 
Example:

Product name: Yamaha YPT270 Tragbares Keyboard mit 61 Tasten

    Url: https://www.gear4music.de/de/Keyboards-and-Klaviere/Yamaha-YPT270-Tragbares-Keyboard-mit-61-Tasten/3B2O

        Date checked: 2020-11-17 11:01:50

        Price: 149.75 EUR

        Available: True

        Is on a promotion: False

        Screenshot URL: https://screenshots.price2spy.com/271/6ab6e071-ba5c-4dd2-9da1-16560cb4c586

"""